package lab01_solutions;

public class IntegerSet {
    private final int SETSIZE = 101;
    private boolean[] set_elements; // holds a set of numbers from 0 - 100
    
  public IntegerSet () {
    // an empty set, all a[i] are set to 0
      set_elements = new boolean[SETSIZE];
  }

  // A constructor that copies from an existing set.
  public IntegerSet (IntegerSet existingSet) {
      set_elements = new boolean[SETSIZE];
      for(int i=0; i < SETSIZE; i++)
          set_elements[i] = existingSet.set_elements[i];
  }

  public void deleteElement(int i) {
    if ((i >= 0) && (i < set_elements.length))
        set_elements[i] = false;  // set to false
  }

  public void insertElement(int i) {
    if ((i >= 0) && (i < set_elements.length))
        set_elements[i] = true;  // set to true
  }

    // Print a set to System.out
    public void toSetString() {
        System.out.print("{ ");
        
        for (int i=0; i<set_elements.length; i++) {
            if (set_elements[i])
                System.out.print( i + " " );
        }
        
        System.out.print("}\n");
    }
    
    // return true if two sets are equal
    public boolean isEqualTo(IntegerSet is2) {
        for(int i=0; i<set_elements.length; i++) {
            if (is2.set_elements[i] != set_elements[i])
                return false;
        }
        return true;
    }
    
  
  // The union of this set and another set
  public IntegerSet union(IntegerSet is2) {
    IntegerSet unionSet = new IntegerSet();
    // unionSet is now an empty set

    for(int i=0; i < SETSIZE; i++) {
        unionSet.set_elements[i] = this.set_elements[i] || is2.set_elements[i]; //if one is true then i belong to the union
    }
    return unionSet;
  }

  // The intersection of this set and another set
  public IntegerSet intersection(IntegerSet is2) {
      IntegerSet intersectionSet = new IntegerSet();
      // intersectionSet is now an empty set
      
      for(int i=0; i < SETSIZE; i++) {
          intersectionSet.set_elements[i] = this.set_elements[i] && is2.set_elements[i]; //if both sets contain i then i belong to intersection
      }
      return intersectionSet;
  }
  
  
  // return true if the set has no elements
  public boolean isEmpty() {
    for (int i=0; i<set_elements.length; i++)
      if (set_elements[i]) 
          return false;
    return true;
  }

  // return the 'length' of a set
  public int cardinality() {
    int count = 0;
    for (int i=0; i<set_elements.length; i++)
      if (set_elements[i])
        count++;
    return count;
  }

  // ------------------------------------------------------------
  // Small test program to verify that IntegerSet works!
  public static void main (String [] args) {
    IntegerSet set1 = new IntegerSet();
    IntegerSet set2 = new IntegerSet();
      java.util.Random r = new java.util.Random();
      
      for (int i=0; i<10; i++) {
          set1.insertElement(r.nextInt(100));
      }
      for (int i=0; i<15; i++) {
          set2.insertElement(r.nextInt(100));
      }
      
      System.out.print("Set 1 :");
      set1.toSetString();

      System.out.print("\nSet 2 :");
      set2.toSetString();

      IntegerSet union = set1.union(set2);
      System.out.print("\nUnion: ");
      union.toSetString();

      IntegerSet intersection = set2.intersection(set1);
      System.out.print("\nIntersection: ");
      intersection.toSetString();
      System.out.println("\n");
  }

    
    /* example output 
     
     Set 1 :{ 5 13 23 43 68 72 74 77 83 }
     
     Set 2 :{ 16 19 23 26 40 49 51 54 58 71 79 82 86 88 98 }
     
     Union: { 5 13 16 19 23 26 40 43 49 51 54 58 68 71 72 74 77 79 82 83 86 88 98 }
     
     Intersection: { 23 }
     
     Compile from console using: 
     $ javac -d . IntegerSet.java
     
     Run from console:
     $ java epl231.lab2012.lab1.IntegerSet
     */
}

